/* eslint-env node, browser */
const fs = require('fs');
const path = require('path');
const browserScript = function(proxyPort, proxyTargets) {
    const readyMessage = 'fetchRecorder:serviceWorker:ready';
    const loadedMessage = 'fetchRecorder:serviceWorker:loaded';
    const info = {
        port: proxyPort,
        targets: proxyTargets
    };
    navigator.serviceWorker.register('/fetchRecorder_serviceWorker.js').then(({ active: serviceWorkerActive }) => {
        if(serviceWorkerActive) {
            navigator.serviceWorker.controller.postMessage(info);
        }
    });
    navigator.serviceWorker.addEventListener('message', ({ data: message }) => {
        if(message === loadedMessage) {
            navigator.serviceWorker.controller.postMessage(info);
        } else if(message === readyMessage) {
            const fetchRecorderReady = new Event(readyMessage);
            document.dispatchEvent(fetchRecorderReady);
        }
    });
};

module.exports = {
    generateScript() {
        const serverInfo = require('fetch-recorder/fetchRecorder_server').getInfo();
        const scriptText = `(${browserScript.toString()})(${serverInfo.port}, ${JSON.stringify(serverInfo.targets)});`;
        const scriptPath = path.join(__dirname, 'fetchRecorder_generatedBrowserScript.js');
        fs.writeFileSync(scriptPath, scriptText);
        return scriptPath;
    }
};
