/* eslint-env worker */
const readyMessage = 'fetchRecorder:serviceWorker:ready';
const loadedMessage = 'fetchRecorder:serviceWorker:loaded';

let info = null;
let basicAuthValue = null;

self.addEventListener('install', (event) => {
    event.waitUntil(self.skipWaiting());
});
self.addEventListener('activate', (event) => {
    const claimClient = self.clients.claim();
    event.waitUntil(claimClient);
    claimClient.then(() => self.clients.matchAll({ includeUncontrolled: true, type: 'window' })).then((clients) => {
        clients.map((client) => client.postMessage(loadedMessage));
    });
});
self.addEventListener('message', ({ data }) => {
    if('port' in data && 'targets' in data) {
        info = {
            port: data.port,
            targets: data.targets.map((url) => new URL(url).origin)
        };
        self.clients.matchAll({ includeUncontrolled: true, type: 'window' }).then((clients) => {
            clients.map((client) => client.postMessage(readyMessage));
        });
    } else if('basicAuthValue' in data) {
        basicAuthValue = data.basicAuthValue;
    }
});
self.addEventListener('fetch', (event) => {
    if(info === null) {
        event.respondWith(fetch(event.request));
        return;
    }
    const request = event.request;
    const requestUrl = new URL(request.url);
    if(info.targets.includes(requestUrl.origin)) {
        const proxyTarget = `fetch-recorder-target=${encodeURIComponent(requestUrl.origin)}`;
        requestUrl.search += requestUrl.search.length ? `&${proxyTarget}` : proxyTarget;
        const newUrl = requestUrl.toString().replace(requestUrl.origin, `https://localhost:${info.port}`);
        const newHeaders = new Headers();
        for(const [ key, value ] of request.headers.entries()) {
            newHeaders.set(key, value);
        }
        if(basicAuthValue !== null) {
            newHeaders.set('authorization', basicAuthValue);
        }
        const newRequest = new Request(newUrl, {
            credentials: request.credentials,
            headers: newHeaders,
            integrity: request.integrity,
            method: request.method,
            mode: request.mode,
            redirect: request.redirect,
            referrer: request.referrer
        });
        if(request.method !== 'GET' && request.method !== 'HEAD') {
            event.respondWith(request.clone().text().then((body) => {
                if(body !== '') {
                    newRequest.body = body;
                }
                return fetch(newRequest);
            }));
        } else {
            event.respondWith(fetch(newRequest));
        }
    } else {
        event.respondWith(fetch(event.request));
    }
});
